## S01: vecteurs numeriques et caracteres

c(1, 3, 4, 10, 2231)

c("dna", "prot", "rna")

#Outre les vecteurs numériques et de caractère, on introduit les vecteurs logiques ou booléens
#ces vecteurs sont composés de 2 éléments possibles, TRUE ou FALSE (valeurs logiques)

vec_l <- c(TRUE, FALSE, TRUE, FALSE)

#comme vu auparavant, la fonction class() permet de renseigner la classe du vecteur
class(vec_l)

TRUE
FALSE

#ainsi, tout vecteur est un élément linéaire qui se compose de valeurs numériques, logiques, ou de caractères.
#pour rappel, tout vecteur ne peut comprendre qu'un seul type d'élément
#Si un vecteur comprend des valeurs numériques et des caractères, le tout est considéré comme caractère
c(1, 2, "3")

#Les valeurs logiques sont de même converties en valeurs numériques : TRUE devient 1, FALSE 0.
c(1, 2, TRUE, FALSE)

#un vecteur mixte comme cela, tout devient caractère.
c(1, 2, TRUE, "FALSE")

#Soit w
w <- c(42, 76, 55, 121, 78)
#PR, length() donne la longueur (ici 5)
length(w)

#PR, on peut choisir comme ceci les premier et troisième éléments:
w[c(1, 3)]

#le même résultat peut être obtenu par des valeurs booléennes: TRUE indique les valeurs désirées
#cette manière est toutefois moins pratique
w[c(TRUE, FALSE, TRUE, FALSE, FALSE)]

## extraire les elements de w plus petits que 60
#cela correspond aux premier et troisième éléments
w[w < 60]

#Ces différentes manières mènent toutes au même résultat
#un vecteur numérique de longueur 2 comprenant 42 et 55
#la réponse est la même, mais pour des questions complètement différentes !

## 1. 
#Soit le vecteur sélection sel composée des valeurs de w < 60
sel <- w < 60
#On obtient le vecteur logique TRUE FALSE TRUE FALSE FALSE
#On extrait ensuite les valeurs de w répondant à cette condition
w[sel]

length(sel)
length(w)
#Par définition, la longueur de sel et w est identique (w utilisé pour créer sel)
#en effet, le vecteur booléen sel est de même longueur que le vecteur w

#Extraire les valeurs de w supérieures à 100
w[w > 100]

## recycling
#Soit le vecteur de sélection, mais dont la longueur ne correspond pas à w
w[c(TRUE, FALSE)]
#on obtient le premier élément (TRUE), pas la deuxième ; après, le vecteur de sélection est répété
#Il est donc répété (recycling) comme TRUE FALSE TRUE FALSE TRUE pour obtenir la longueur désirée
w[c(TRUE, FALSE, TRUE, FALSE, TRUE)]
#On obtient donc les éléments 1, 3 et 5

#le : permet de créer facilement un vecteur numériques de valeurs comprises entre les deux nombres entrés
1:10
1:5
#On peut sommer de tels vecteurs
(1:2) + (1:2)
#la somme de deux vecteurs de longueurs différentes mène également au recycling (second vecteur répété)
(1:10) + (1:5)

#La commande (opérateur) == permet de confirmer qqch
#est-ce que les longueurs de w et sel sont égales ? TRUE
length(w)==length(sel)
length(sel) == length(w)

## Autre opérateur logique : ! negation

!TRUE
#la négation de TRUE est FALSE
!FALSE
#idem

## elements inferieurs ou egaux a 100 : deux manières de répondre
w[w <= 100]
w[!w > 100]

## autre opérateur : ou: | 
## dernier opérateur : et: &
#permettent de combiner des conditions: ici éléments inférieurs à 60 OU supérieurs à 100 
w < 60
w > 100
w[w < 60 | w > 100]
#les deux conditions génèrent chacune un vecteur booléens de même longueur
#la combinaison des deux conditions génère un vecteur booléen tjs de même longueur

w > 60
w < 100
w[w > 60 & w < 100]
#idem

#la combinaison de deux conditions 'exclusives' donne le vecteur vide: numeric(0)
vec_vide <- w[w < 60 & w > 100]
length(vec_vide)
#de longueur nulle
class(vec_vide)
#de classe numérique

##EXERCICE en partant des du vecteur w, creez w2 qui contient 
##les elements strictement compris entre 75 et 100
w <- c(42, 76, 55, 121, 78)

w < 75 
w > 100
#combinaison logique donnée par
w2 <- w[w > 75 & w < 100]
#vecteur de même longueur que le vecteur w

sel <- w > 75 & w < 100
#sel est un vecteur booléen de longueur égale à w
#la fonction which renseigne sur quels éléments sont TRUE par leurs indices
which(sel)
w[sel]
w[which(sel)]


#Soit le vecteur de caractère x
x <- c("dna", "prot", "rna")
#pour obtenir tous les éléments dna ou rna
x[x== "dna" | x== "rna"]

x[x == "dna" | x == "rna" | x == "meta" | x == "autre"]
#par facilité, on peut réécrire cela en ;
x[x %in% c("dna", "rna", "meta", "autre")]
# %in% est "quels éléments de x sont dans (le vecteur c("dna", "rna", etc.) ici)



## names / noms des éléments de vecteurs

w
names(w)
#pas de nom pour les valeurs de w: NULL

#On assigne manuellement des noms aux valeurs de w par le signe <- avec des guillemets
names(w) <- c("a", "b", "c", "d", "e")

names(w)
#noms de w sont a, b, c, d et e

w[c(1, 3)]
#On peut extraire des valeurs particulières de w indépendamment de leur position mais par leur nom:
w[c("a", "c")]

#En ccl, subsetting ainsi possible par l'index, le nom ou les valeurs logiques.
## subsetting:
## - index
## - noms
## - logiques

names(w) <- LETTERS[1:5]
w

#Il faut que les noms assignés correspondent à une séquence prédéfinie pour nommer automatiquement;
pi
letters
LETTERS
month.abb
month.name

names(w) <- c("a12", "z3", "c1", "a44", "a00")
#Les noms ici n'ont pas d'ordre logique et doivent être assignés manuellement.

## donnees manquantes - missing data

names(w) <- c("a", "b", "c", "d", "e")
w[c("a", "c", "z")]
#z est marqué comme NA = not available = donnée manquante : pas d'élément nommé z dans w
#NA décrit ainsi l'absence de données, une donnée manquante



#Soit le vecteur contenant des données manquantes
x <- c(1, 4, 6, 3, NA, 6)
x
length(x)
#longueur 6 : NA est donc bien une valeur mais manquante
length(c(NA, NA, NA))
class(x)
#la classe de x reste numérique : NA s'adapte à la classe des autres éléments
class(c("A", "a", NA))

mean(x)
#la moyenne donne NA : la moyenne ne peut être calculée avec une valeur NA !

#Il faut paramétriser la fonction mean() en enlevant la valeur manquante NA
#par défaut, les valeurs manquantes ne sont pas enlevées = FALSE
mean(x, na.rm = FALSE)
#les NA sont éliminées pour le calcul de la moyenne
mean(x, na.rm = TRUE)

#Idem pour le calcul de la somme : les NA doivent être enlevées
sum(x, na.rm = TRUE)

#la fonction is.na() retourne les booléens pour trouver les valeurs manquantes
is.na(x)
#which() extrait quels éléments sont NA
which(is.na(x))

#On peut utiliser la négation ! pour trouver quels éléments ne sont PAS manquants
!is.na(x)
#idem, which() pour connaître l'indice desdits éléments
which(!is.na(x))

#On crée un nouveau x qui ne contient pas de NA
x[!is.na(x)]
#Ce vecteur peut donc être utilisé pour calculer la moyenne de x, sans NA : de deux manières;
mean(x[!is.na(x)])
mean(x, na.rm = TRUE)

#la fonction na.omit() permet d'extraire les NA:
na.omit(x)

#la fonction complete.cases() indique quels éléments ne sont pas NA:
x[complete.cases(x)]

#EXERCICE: soit le vecteur de hauteur heights de longueur 21
heights <- c(63, 69, 60, 65, NA, 68, 61, 70, 61, 59, 64, 69, 63, 63, NA, 72, 65, 64, 70, 63, 65)
length(heights)

## Using this vector of heights in inches, 
## create a new vector with the NAs removed.
#Y a-t-il dans ce vecteur booléen des valeurs vraies?
any(is.na(heights))
#la somme des deux vecteurs booléens donne 2 (TRUE = 1)
sum(is.na(heights))
#Quelles positions sont manquantes ?
which(is.na(heights))
#la fonction table() compte les TRUE et FALSE (pas nécessaire)
table(is.na(heights))
#On extrait les NA par na.omit()
na.omit(heights)
#Autre manière
h2 <- heights[!is.na(heights)]
h2

## Use the function median() to calculate the median of
## the heights vector.
#deux manières
median(heights, na.rm = TRUE)
median(h2)

## Use R to figure out how many people in the set are taller 
## than 67 inches.

heights > 67 ## ! NA
#la somme donne NA
sum(heights > 67)
#il faut extraire les NA
sum(heights > 67, na.rm = TRUE)
#6 personnes ont une taille supérieure à 67
#Autre manière
sum(na.omit(heights) > 67)
#Autre manière
sum(h2 > 67)

#GENERATION AUTOMATIQUE DE VECTEURS (NON PLUS MANUELLE)
#la fonction numeric() crée un vecteur de longueur entrée et de valeurs nulles
numeric(10)

#la fonction rep() répète un certain vecteur de valeurs première entrée et de longueur seconde entrée
rep(1, 10)
rep(NA, 10)
#on peut répéter un vecteur de longueur donnée
rep(c(1, 4, 10), 10)
#on peut répéter le premier élement x fois, le second x fois, le troisième x fois, etc.
rep(c(1, 4, 10), each = 10)
rep(letters[1:5], 3)

#sec() permet de créer un vecteur de séquence en entrant l'élément de départ, le dernier, et le pas.
seq(1, 10, 2)
seq(1, 10, 3)
#Autre notation
seq(from = 1, to = 10, by = 3)
#length.out indique la longueur désirée du vecteur comprenant les valeurs initiales et finales désirées
seq(1, 10, length.out = 4)
seq(1, 10, length.out = 3)

#PR Autre notation pour le vecteur 1 2 3 4 5 6 7 8 9 10
1:10
seq(1, 10, 1)

#paste() colle les valeurs entrées en un nouveau caractère (ici a1)
paste("a", "1", sep = "")
#elle peut être combinée avec les letters pour obtenir a1, b2, c3, etc.
names(w) <- paste(letters[1:5], 1:5, sep = "")
w

#sample() génère un échantillon aléatoire en permutant les entrées
sample(10)
#Autre notation
sample(1:10)
sample(letters)
#obtenir un échantillon aléatoire dans un intervalle défini:
sample(10, 5)
#Si la longueur est supérieure au nombre de valeurs, il faut indiquer replace = TRUE
#On obtient plusieurs même éléments alors (permutation avec remplacement)
sample(1:10, 20, replace = TRUE)

#rnorm() échantillone x éléments d'une distribution normale de moyenne 0 et écart-type 1 par défaut
rnorm(10)
#Autre notation mais même chose:
rnorm(10, mean = 0, sd = 1)
rnorm(100000)
#la moyenne et l'écart-type peuvent être modifiées pour modifier la distribution
rnorm(100000, 10, 2)
mean(rnorm(10000, mean = 0, sd = 1))
#la moyenne est proche de 0 (mais pas égale car l'échantillonage est aléatoire)
mean(rnorm(10000, mean = 10, sd = 2))
#idem avec 10

#EXERCICE
## Create two vectors x and y containing the numbers 1 to 10 and 
## 10 to 1 respectively. You can use the seq or : functions rather 
## than constructing them by hand.

x <- seq(1, 10)
x
y <- seq(10, 1)
y

## Check their type. Depending how they were created, 
## they can be integers (entier) or doubles.
class(x)
class(y)

## Take the sum (see the sum() function) of each vector 
## and verify they are identical.
sum(x)
sum(y)
#Ces sommes sont-elles identiques ?
sum(x) == sum(y)
#autre manière avec identical()
identical(sum(x), sum(y))

## Sum vectors element-wise (éléments par éléments), and verify that all results are identical.
#Vérifier que les deux vecteurs x et y ont la même longueur
length(x) == length(y)
x
y
x + y
z <- x + y
z
length(z)
#longueur du vecteur somme est égale au vecteur le plus long (même si x et y ont la même longueur)

#unique() permet d'éliminer les doublons d'un vecteur
unique(z)
length(unique(z)) == 1

## Swap the value or x and y (inverser le contenu de ces deux variables).
#Pour cela, on passe par une valeur temporaire
x
y

x_old <- x
x <- y
y <- x_old
#effacer la variable temporaire
rm(x_old)

#rev() permet de renverser les valeurs du vecteur (commencer par la fin)
rev(x)
rev(y)
